﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.IO;

namespace NWNHeatMap
{
    public partial class HeatmapForm : Form
    {
        private Bitmap bmp = null;
        private Image area = null;
        private List<HeatMapPoint> points = null;

        public HeatmapForm()
        {
            InitializeComponent();
            System.Drawing.Size minsize = new System.Drawing.Size { Width = this.Width, Height = this.Height };
            this.MinimumSize = minsize;
        }

        private void panelImage_Paint(object sender, PaintEventArgs e)
        {
            if (bmp != null)
            {
                Graphics g = e.Graphics;
                g.DrawImage(bmp, 0, 0, bmp.Width, bmp.Height);
                g.Dispose();
            }
        }

        private void tkbarOpacity_Scroll(object sender, EventArgs e)
        {
            numOpacity.Value = (decimal)tkbarOpacity.Value / (decimal)100.0;
        }

        private void tkbarDotradius_Scroll(object sender, EventArgs e)
        {
            numDotradius.Value = tkbarDotradius.Value;
        }

        private void tkbarScaling_Scroll(object sender, EventArgs e)
        {
            numScaling.Value = (decimal)tkbarScaling.Value / (decimal)10.0;
        }

        private void numScaling_ValueChanged(object sender, EventArgs e)
        {
            tkbarScaling.Value = (int)((decimal)10.0 * numScaling.Value);
        }

        private void numDotradius_ValueChanged(object sender, EventArgs e)
        {
            tkbarDotradius.Value = (int)numDotradius.Value;
        }

        private void numOpacity_ValueChanged(object sender, EventArgs e)
        {
            tkbarOpacity.Value = (int)((decimal)100.0 * numOpacity.Value);
        }

        private void bUpdate_Click(object sender, EventArgs e)
        {
            if (area == null || points == null)
            {
                MessageBox.Show("You must load an image and dataset before generating a heatmap.");
                return;
            }

            HeatMap m = new HeatMap(area, (float)numScaling.Value,
                (float)numDotradius.Value, (float)numOpacity.Value,
                cbGrayscale.Checked, cbTrimheatless.Checked);

            foreach (HeatMapPoint p in points)
            {
                m.AddPoint(p.X, p.Y);
            }

            bmp = m.GetBitmap();
            panelImage.Invalidate();
        }

        private void bSave_Click(object sender, EventArgs e)
        {
            if (bmp == null)
            {
                MessageBox.Show("You must generate a heatmap before you can save one!");
                return;
            }

            sfdImage.ShowDialog();
            try
            {
                // Save the finished image
                FileStream fs = new FileStream(sfdImage.FileName, FileMode.Create);
                bmp.Save(fs, ImageFormat.Png);
                fs.Close();
            }
            catch
            {
                MessageBox.Show("Failed to save?! :(");
            }
        }

        private void bOpenImage_Click(object sender, EventArgs e)
        {
            ofdImage.ShowDialog();
            try
            {
                this.area = Image.FromFile(ofdImage.FileName);
                string[] pathsplit = ofdImage.FileName.Split('\\');
                this.lImgLoaded.Text = "Image Loaded: " + pathsplit[pathsplit.Length - 1];
            }
            catch
            {
                MessageBox.Show("Failure loading image file.");
            }
        }

        private void bOpenDataset_Click(object sender, EventArgs e)
        {
            ofdDataset.ShowDialog();
            try
            {
                StreamReader sr = new StreamReader(ofdDataset.FileName);
                points = new List<HeatMapPoint>();
                string ln;
                string[] nums;
                float x, y;
                while ((ln = sr.ReadLine()) != null)
                {
                    nums = ln.Split('\t');
                    x = float.Parse(nums[0].Trim('\"'));
                    y = float.Parse(nums[1].Trim('\"'));
                    // This is kind of an abuse of HeatMapPoints, but it serves its purpose.
                    points.Add(new HeatMapPoint(x, y));
                }
                string[] pathsplit = ofdDataset.FileName.Split('\\');
                this.lDataLoaded.Text = "Dataset Loaded: " + pathsplit[pathsplit.Length - 1];
            }
            catch
            {
                MessageBox.Show("Failure loading dataset.");
            }
        }
    }
}
